# schichtleiter
# Copyright (C) 2022  Luca Schmid
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from http.server import BaseHTTPRequestHandler, HTTPServer
from icalendar import Calendar
from schichtleiter.schedule import calendar_to_xml
import requests
import sys

def run_server(address, port, urls):
    class RequestHandler(BaseHTTPRequestHandler):
        def do_GET(self):
            url = self.path.lstrip('/')
            if url not in urls:
                self.send_error(404)
                return

            try:
                r = requests.get(url)
                r.raise_for_status()
            except:
                self.send_error(502)
                raise

            if not r.headers['content-type'].startswith('text/calendar'):
                self.send_error(502)
                self.log_error('upstream response is not a calendar')
                return

            try:
                cal = Calendar.from_ical(r.text)
            except:
                self.send_error(502)
                raise

            try:
                schedule_xml = calendar_to_xml(cal)
            except:
                self.send_error(500)
                raise

            self.send_response(200)
            self.send_header('Content-Type', 'application/xml')
            self.end_headers()

            # schedule_xml is a utf-8-encoded bytes object
            self.wfile.write(schedule_xml)

    httpd = HTTPServer((address, port), RequestHandler)
    print(f'Listening on {address}:{port} ...', file=sys.stderr)
    httpd.serve_forever()
